#!bin/bash
#Variablendeklarationen
sharePath="/usr/share/mzf-HA-base"
corosyncPath="/etc/corosync"
pacemakerPath="/etc/pacemaker"
drbdPath="/etc/drbd.d"
dlmPath="/etc/dlm"
tmpFolder="/tmp/haConfigFiles"

clusterName=""
minNodeCount=2
minRingCount=1
nodeCount=0
ringCount=0
currentHostname=""
hostnames=()
netAddresses=()
ipAddresses=()
drbdIpAddresses=()
corosyncPortNrs=()
drbdPortNr=""
drbdDisks=()
drbdDevice=""
file=""
fileContent=""


# $1 = NodeNumber, 
# $2 = RingCount, 
# $3 = RingIndex, 
# $4 = value 
function SetIpAddressArray() {
	ipAddresses[($1 * $2) + $3]+=$4
}

# $1 = NodeNumber,
# $2 = RingCount, 
# $3 = RingIndex,
# $4 = returnValue
function GetIpAddress() {
	ret=${ipAddresses[($1 * $2) + $3]}
}

#Get NodeCount
while [ $nodeCount -lt $minNodeCount ]; do
	read -p "Anzahl der Knoten: " nodeCount
done

#Get ClusterName
while [ "$clusterName" == "" ]; do
	read -p "Clustername: " clusterName
done

#Get hostnames
for ((i = 0; i < $minNodeCount; i++)) do
	read -p "Hostname Knoten $(($i+1)): " inp
	hostnames[i]+=$inp
done

#Get Hostname for current machine
echo "Hostnamen für diesen Knoten festlegen: ";
for((i = 0; i < ${#hostnames[@]}; i++)) do
	echo "${hostnames[$i]}[$i]";
done
read -p "Ihre Auswahl (0-$(($i-1))): " currentHostname

#Get RingCount
while [ $ringCount -lt $minRingCount ]; do
	read -p "Anzahl der Cluster-Kommunikationspfade (Rings): " ringCount
done

#Get RingInfos
for((i = 0; i < $ringCount; i++)) do
	read -p "Netz-Adresse für Ring $i: " inp
	netAddresses[i]+=$inp
	read -p "Portnr. für Ring $i: " inp
	corosyncPortNrs[i]+=$inp
done

#Get NodeRingAddresses
for((i = 0; i < $nodeCount; i++)) do
	for((j = 0; j < $ringCount; j++)) do
		read -p "Ring $j - Adresse für ${hostnames[$i]}: " inp 
		SetIpAddressArray $i $ringCount $j $inp
	done
done

#Get DRBD-Device
read -p "DRBD-Device: " inp
drbdDevice+=$inp

#Get DRBD-Disks
for((i = 0; i < $nodeCount; i++)) do
	read -p "DRBD-Disk für ${hostnames[i]}: " inp
	drbdDisks[i]+=$inp
done

#Get DRBD-IP-Addresses
echo -e "Netzwerk für DRBD-Replikation:"
for((i = 0; i < $ringCount; i++)) do
	echo -e "\t${netAddresses[$i]} [$i]"
done
read -p "Ihre Auswahl (0-$(($ringCount - 1))): " inp

for((i = 0; i < $nodeCount; i++)) do 
	GetIpAddress $i $ringCount $inp
	drbdIpAddresses[i]+=$ret
done

#Get DRBD-Port
read -p "Port für DRBD-Replikation: " inp
drbdPortNr=$inp

#Create tmpFolder
mkdir -p "$tmpFolder"

######################################################################################################################################
#Generate corosync.conf
######################################################################################################################################
file="$tmpFolder/corosync.conf"

#Create File
touch "$file"

#Generate Totem-Section
echo -e "totem {\n\tversion: 2\n\tcluster_name: $clusterName" > $file
if [ $ringCount -gt 1 ] 
then
	echo -e "\trrp_mode: passive" >> $file
fi
for((i = 0; i < ringCount; i++)) do
	echo -e "\tinterface {\n\t\tringnumber: $i\n\t\tbindnetaddr: ${netAddresses[i]}\n\t\tmcastport: ${corosyncPortNrs[i]}\n\t}" >> $file
done
echo -e "\n}\n" >> $file

#Generate NodeList
echo -e "nodelist {" >> $file
for((i = 0; i < $nodeCount; i++)) do
	echo -e "\tnode {\n\t\tnodeid: $(($i+1))" >> $file
	for((j = 0; j < $ringCount; j++)) do
		GetIpAddress $i $ringCount $j
		echo -e "\t\tring$(($j))_addr: $ret" >> $file
	done
	echo -e "\t}" >> $file
done
echo -e "}\n" >> $file

#Generate Quorum
echo -e "quorum {\n\tprovider: corosync_votequorum" >> $file
if [ $nodeCount -eq 2 ] 
then
	echo -e "\ttwo_node: 1" >> $file
fi
echo -e "}\n" >> $file

#Generate Service
echo -e "service {\n\tver: 1\n\tname: pacemaker\n}" >> $file

#Generate logging
echo -e "logging {\n\tto_logfile: yes\n\tlogfile: /var/log/corosync/corosync.log\n\tto_syslog: yes\n\ttimestamp: on\n}" >> $file

######################################################################################################################################
# End Generate corosync.conf
######################################################################################################################################

######################################################################################################################################
# Generate drbd.res
######################################################################################################################################
file="$tmpFolder/$clusterName.res"

#Create File
touch "$file"

#Generate Globals
echo -e "resource $clusterName {\n\tprotocol C;" > $file

#Generate syncer
echo -e "\tsyncer {\n\t\trate 1G;\n\t\tal-extents 3389;\n\t\tverify-alg sha1;\n\t}" >> $file

#Generate net
echo -e "\tnet {\n\t\tcram-hmac-alg sha1;\n\t\tshared-secret \"metasec\";\n\t\tallow-two-primaries;\n\t\tafter-sb-0pri discard-zero-changes;\n\t\tafter-sb-1pri discard-secondary;\n\t\tafter-sb-2pri disconnect;\n\t}" >> $file

#Generate startup
echo -e "\tstartup {\n\t\tbecome-primary-on both;\n\t}" >> $file

#Generate disk
echo -e "\tdisk {\n\t\ton-io-error detach;\n\t\tfencing resource-only;\n\t}" >> $file

#Generate handlers
echo -e "\thandlers {\n\t\tfence-peer \"/usr/lib/drbd/crm-fence-peer.sh\";\n\t\tafter-resync-target \"/usr/lib/drbd/crm-unfence-peer.sh\";\n\t}" >> $file

#Generate device specific ressources
for((i = 0; i < $nodeCount; i++)) do
	echo -e "\ton ${hostnames[i]} {\n\t\tdevice $drbdDevice;\n\t\tdisk ${drbdDisks[i]};\n\t\taddress ${drbdIpAddresses[i]}:$drbdPortNr;\n\t\tmeta-disk internal;\n\t}" >> $file
done 

#Generate EndBracket
echo -e "}" >> $file

######################################################################################################################################
# End Generate drbd.res
######################################################################################################################################

######################################################################################################################################
# Copy Files
######################################################################################################################################

for((i = 0; i < $nodeCount; i++)) do
	if [ "$i" -eq "$currentHostname" ]
	then
		mkdir -p "$dlmPath"
		mkdir -p "$drbdPath"
		mkdir -p "$pacemakerPath"
		mkdir -p "corosyncPath"
		mkdir -p "$stonithPath/external/"

		cp -rf "$sharePath/dlm/dlm.conf" "$dlmPath/dlm.conf"
		cp -rf "$sharePath/drbd/drbd.conf" "/etc/drbd.conf"
		cp -rf "$sharePath/drbd/global_common.conf" "$drbdPath/global_common.conf"
		cp -rf "$sharePath/pacemaker/authkey" "$pacemakerPath/authkey"
		cp -rf "$sharePath/pacemaker/sampleConfig.xml" "$pacemakerPath/sampleConfig.xml"
		cp -rf "$sharePath/corosync/authkey" "$corosyncPath/authkey"
		cp -rf "$sharePath/stonith/node1/ssh" "$stonithPath/external/ssh"

		cp -rf "$tmpFolder/corosync.conf" "$corosyncPath/corosync.conf"
		cp -rf "$tmpFolder/$clusterName.res" "$drbdPath/$clusterName.res"
	else
		ssh "root@${hostnames[i]}" "mkdir -p $dlmPath | mkdir -p $drbdPath | mkdir -p $pacemakerPath | mkdir -p corosyncPath | mkdir -p $stonithPath/external/"
		scp -r "$sharePath/dlm/dlm.conf" "root@${hostnames[i]}:$dlmPath/dlm.conf"
		scp -r "$sharePath/drbd/drbd.conf" "root@${hostnames[i]}:/etc/drbd.conf"
		scp -r "$sharePath/drbd/global_common.conf" "root@${hostnames[i]}:$drbdPath/global_common.conf"
		scp -r "$sharePath/pacemaker" "root@${hostnames[i]}:$pacemakerPath"
		scp -r "$sharePath/corosync/authkey" "root@${hostnames[i]}:$corosyncPath/authkey"
		scp -r "$sharePath/stonith/node2/ssh" "root@${hostnames[i]}:$stonithPath/external/ssh"

		scp -r "$tmpFolder/corosync.conf" "root@${hostnames[i]}:$corosyncPath/corosync.conf"
		scp -r "$tmpFolder/$clusterName.res" "root@${hostnames[i]}:$drbdPath/$clusterName.res"
	fi
done

#Tidy up
rm -r "$tmpFolder"

######################################################################################################################################
# End Copy Files
######################################################################################################################################

######################################################################################################################################
# Configure Programs
######################################################################################################################################

#restart corosync

#Configure drbd
if [ "$nodeCount" -eq 2 ]
then
	service "corosync" "restart"
	drbdadm "create-md" "all"
	if [ "$currentHostname" -eq 0 ]
	then
		ssh "root@${hostnames[1]}" "service corosync restart" 
		ssh "root@${hostnames[1]}" "drbdadm create-md all" 
		service "drbd" "start" | ssh "root@${hostnames[1]}" "service drbd start"
	else
		ssh "root@${hostnames[0]}" "service corosync restart"
		ssh "root@${hostnames[0]}" "drbdadm create-md all" 
		service "drbd" "restart" | ssh "root@${hostnames[0]}" "service drbd restart"
	fi
	drbdadm "--" "--overwrite-data-of-peer" "primary" "$clusterName"
fi

######################################################################################################################################
# End Configure Programs
######################################################################################################################################
